@@ -90,8 +90,23 @@ namespace dlib
             _at_start(true)
         {}
 
+        array(
+            array&& item
+        ) : array()
+        {
+            swap(item);
+        }
+
+        array& operator=(
+            array&& item
+        )
+        {
+            swap(item);
+            return *this;
+        }
+
         explicit array (
-            unsigned long new_size
+            size_t new_size
         ) :
             array_size(0),
             max_array_size(0),
@@ -110,22 +125,22 @@ namespace dlib
         );
 
         inline const T& operator[] (
-            unsigned long pos
+            size_t pos
         ) const;
 
         inline T& operator[] (
-            unsigned long pos
+            size_t pos
         );
 
         void set_size (
-            unsigned long size
+            size_t size
         );
 
-        inline unsigned long max_size(
+        inline size_t max_size(
         ) const;
 
         void set_max_size(
-            unsigned long max
+            size_t max
         );
 
         void swap (
@@ -133,7 +148,7 @@ namespace dlib
         );
 
         // functions from the enumerable interface
-        inline unsigned long size (
+        inline size_t size (
         ) const;
 
         inline bool at_start (
@@ -158,7 +173,7 @@ namespace dlib
         );
 
         void resize (
-            unsigned long new_size
+            size_t new_size
         );
 
         const T& back (
@@ -178,6 +193,10 @@ namespace dlib
             T& item
         );
 
+        void push_back (
+            T&& item
+        );
+
         typedef T* iterator;
         typedef const T* const_iterator;
         iterator                begin()                         { return array_elements; }
@@ -190,8 +209,8 @@ namespace dlib
         typename mem_manager::template rebind<T>::other pool;
 
         // data members
-        unsigned long array_size;
-        unsigned long max_array_size;
+        size_t array_size;
+        size_t max_array_size;
         T* array_elements;
 
         mutable T* pos;
@@ -229,7 +248,7 @@ namespace dlib
             serialize(item.max_size(),out);
             serialize(item.size(),out);
 
-            for (unsigned long i = 0; i < item.size(); ++i)
+            for (size_t i = 0; i < item.size(); ++i)
                 serialize(item[i],out);
         }
         catch (serialization_error e)
@@ -249,12 +268,12 @@ namespace dlib
     {
         try
         {
-            unsigned long max_size, size;
+            size_t max_size, size;
             deserialize(max_size,in);
             deserialize(size,in);
             item.set_max_size(max_size);
             item.set_size(size);
-            for (unsigned long i = 0; i < size; ++i)
+            for (size_t i = 0; i < size; ++i)
                 deserialize(item[i],in);
         }
         catch (serialization_error e)
@@ -314,7 +333,7 @@ namespace dlib
         >
     const T& array<T,mem_manager>::
     operator[] (
-        unsigned long pos
+        size_t pos
     ) const
     {
         // make sure requires clause is not broken
@@ -337,7 +356,7 @@ namespace dlib
         >
     T& array<T,mem_manager>::
     operator[] (
-        unsigned long pos
+        size_t pos
     ) 
     {
         // make sure requires clause is not broken
@@ -360,7 +379,7 @@ namespace dlib
         >
     void array<T,mem_manager>::
     set_size (
-        unsigned long size
+        size_t size
     )
     {
         // make sure requires clause is not broken
@@ -386,7 +405,7 @@ namespace dlib
         typename T,
         typename mem_manager
         >
-    unsigned long array<T,mem_manager>::
+    size_t array<T,mem_manager>::
     size (
     ) const
     {
@@ -401,7 +420,7 @@ namespace dlib
         >
     void array<T,mem_manager>::
     set_max_size(
-        unsigned long max
+        size_t max
     )
     {
         reset();
@@ -439,7 +458,7 @@ namespace dlib
         typename T,
         typename mem_manager
         >
-    unsigned long array<T,mem_manager>::
+    size_t array<T,mem_manager>::
     max_size (
     ) const
     {
@@ -457,8 +476,8 @@ namespace dlib
         array<T,mem_manager>& item
     )
     {
-        unsigned long    array_size_temp        = item.array_size;
-        unsigned long    max_array_size_temp    = item.max_array_size;
+        auto             array_size_temp        = item.array_size;
+        auto             max_array_size_temp    = item.max_array_size;
         T*               array_elements_temp    = item.array_elements;
 
         item.array_size         = array_size;
@@ -627,7 +646,7 @@ namespace dlib
         >
     void array<T,mem_manager>::
     resize (
-        unsigned long new_size
+        size_t new_size
     )
     {
         if (this->max_size() < new_size)
@@ -635,7 +654,7 @@ namespace dlib
             array temp;
             temp.set_max_size(new_size);
             temp.set_size(new_size);
-            for (unsigned long i = 0; i < this->size(); ++i)
+            for (size_t i = 0; i < this->size(); ++i)
             {
                 exchange((*this)[i],temp[i]);
             }
@@ -750,7 +769,7 @@ namespace dlib
             array temp;
             temp.set_max_size(this->size()*2 + 1);
             temp.set_size(this->size()+1);
-            for (unsigned long i = 0; i < this->size(); ++i)
+            for (size_t i = 0; i < this->size(); ++i)
             {
                 exchange((*this)[i],temp[i]);
             }
@@ -764,6 +783,17 @@ namespace dlib
         }
     }
 
+// ----------------------------------------------------------------------------------------
+
+    template <
+        typename T,
+        typename mem_manager
+        >
+    void array<T,mem_manager>::
+    push_back (
+        T&& item
+    ) { push_back(item); }
+
 // ----------------------------------------------------------------------------------------
 
     template <typename T, typename MM>
