@@ -3,6 +3,7 @@
 #ifndef DLIB_CBLAS_CONSTAnTS_Hh_
 #define DLIB_CBLAS_CONSTAnTS_Hh_
 
+#ifndef CBLAS_H
 namespace dlib
 {
     namespace blas_bindings
@@ -15,6 +16,7 @@ namespace dlib
 
     }
 }
+#endif // if not CBLAS_H
 
 #endif // DLIB_CBLAS_CONSTAnTS_Hh_
 
