@@ -16,6 +16,17 @@ namespace dlib
 // ----------------------------------------------------------------------------------------
 //                                   Simple matrix utilities 
 // ----------------------------------------------------------------------------------------
+// ----------------------------------------------------------------------------------------
+
+    template <typename EXP>
+    constexpr bool is_row_major (
+        const matrix_exp<EXP>&
+    );
+    /*!
+        ensures
+            - returns true if and only if the given matrix expression uses the row_major_layout.
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     const matrix_exp diag (
@@ -1350,6 +1361,34 @@ namespace dlib
               according to std::norm().
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    const matrix_exp min_pointwise (
+        const matrix_exp& a,
+        const matrix_exp& b 
+    );
+    /*!
+        requires
+            - a.nr() == b.nr()
+            - a.nc() == b.nc()
+            - a and b both contain the same type of element 
+        ensures
+            - returns a matrix R such that:
+                - R::type == the same type that was in a and b.
+                - R has the same dimensions as a and b. 
+                - for all valid r and c:
+                  R(r,c) == std::min(a(r,c), b(r,c))
+    !*/
+
+    const matrix_exp min_pointwise (
+        const matrix_exp& a,
+        const matrix_exp& b,
+        const matrix_exp& c 
+    );
+    /*!
+        performs min_pointwise(a,min_pointwise(b,c));
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     const matrix_exp::type max (
@@ -1364,6 +1403,34 @@ namespace dlib
               according to std::norm().
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    const matrix_exp max_pointwise (
+        const matrix_exp& a,
+        const matrix_exp& b 
+    );
+    /*!
+        requires
+            - a.nr() == b.nr()
+            - a.nc() == b.nc()
+            - a and b both contain the same type of element 
+        ensures
+            - returns a matrix R such that:
+                - R::type == the same type that was in a and b.
+                - R has the same dimensions as a and b. 
+                - for all valid r and c:
+                  R(r,c) == std::max(a(r,c), b(r,c))
+    !*/
+
+    const matrix_exp max_pointwise (
+        const matrix_exp& a,
+        const matrix_exp& b,
+        const matrix_exp& c 
+    );
+    /*!
+        performs max_pointwise(a,max_pointwise(b,c));
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     void find_min_and_max (
@@ -1642,7 +1709,7 @@ namespace dlib
         typename T,
         typename P
         >
-    const matrix_exp pixel_to_vector (
+    const matrix<T,pixel_traits<P>::num,1> pixel_to_vector (
         const P& pixel
     );
     /*!
