@@ -1,9 +1,9 @@
 // Copyright (C) 2015 Davis E. King (davis@dlib.net)
 // License: Boost Software License   See LICENSE.txt for the full license.
 
+#include "opaque_types.h"
 #include <dlib/python.h>
 #include <dlib/matrix.h>
-#include <boost/python/args.hpp>
 #include <dlib/geometry.h>
 #include <dlib/image_processing/frontal_face_detector.h>
 #include "simple_object_detector.h"
@@ -12,7 +12,8 @@
 
 using namespace dlib;
 using namespace std;
-using namespace boost::python;
+
+namespace py = pybind11;
 
 // ----------------------------------------------------------------------------------------
 
@@ -26,13 +27,13 @@ string print_simple_test_results(const simple_test_results& r)
 // ----------------------------------------------------------------------------------------
 
 inline simple_object_detector_py train_simple_object_detector_on_images_py (
-    const boost::python::list& pyimages,
-    const boost::python::list& pyboxes,
+    const py::list& pyimages,
+    const py::list& pyboxes,
     const simple_object_detector_training_options& options
 )
 {
-    const unsigned long num_images = len(pyimages);
-    if (num_images != len(pyboxes))
+    const unsigned long num_images = py::len(pyimages);
+    if (num_images != py::len(pyboxes))
         throw dlib::error("The length of the boxes list must match the length of the images list.");
 
     // We never have any ignore boxes for this version of the API.
@@ -44,14 +45,14 @@ inline simple_object_detector_py train_simple_object_detector_on_images_py (
 }
 
 inline simple_test_results test_simple_object_detector_with_images_py (
-        const boost::python::list& pyimages,
-        const boost::python::list& pyboxes,
+        const py::list& pyimages,
+        const py::list& pyboxes,
         simple_object_detector& detector,
         const unsigned int upsampling_amount
 )
 {
-    const unsigned long num_images = len(pyimages);
-    if (num_images != len(pyboxes))
+    const unsigned long num_images = py::len(pyimages);
+    if (num_images != py::len(pyboxes))
         throw dlib::error("The length of the boxes list must match the length of the images list.");
 
     // We never have any ignore boxes for this version of the API.
@@ -65,8 +66,8 @@ inline simple_test_results test_simple_object_detector_with_images_py (
 // ----------------------------------------------------------------------------------------
 
 inline simple_test_results test_simple_object_detector_py_with_images_py (
-        const boost::python::list& pyimages,
-        const boost::python::list& pyboxes,
+        const py::list& pyimages,
+        const py::list& pyboxes,
         simple_object_detector_py& detector,
         const int upsampling_amount
 )
@@ -85,9 +86,9 @@ inline simple_test_results test_simple_object_detector_py_with_images_py (
 // ----------------------------------------------------------------------------------------
 
 inline void find_candidate_object_locations_py (
-    object pyimage,
-    boost::python::list& pyboxes,
-    boost::python::tuple pykvals,
+    py::object pyimage,
+    py::list& pyboxes,
+    py::tuple pykvals,
     unsigned long min_size,
     unsigned long max_merging_iterations
 )
@@ -101,20 +102,20 @@ inline void find_candidate_object_locations_py (
     else
         throw dlib::error("Unsupported image type, must be 8bit gray or RGB image.");
 
-    if (boost::python::len(pykvals) != 3)
+    if (py::len(pykvals) != 3)
         throw dlib::error("kvals must be a tuple with three elements for start, end, num.");
 
-    double start = extract<double>(pykvals[0]);
-    double end   = extract<double>(pykvals[1]);
-    long num     = extract<long>(pykvals[2]);
+    double start = pykvals[0].cast<double>();
+    double end   = pykvals[1].cast<double>();
+    long num     = pykvals[2].cast<long>();
     matrix_range_exp<double> kvals = linspace(start, end, num);
 
     std::vector<rectangle> rects;
-    const long count = len(pyboxes);
+    const long count = py::len(pyboxes);
     // Copy any rectangles in the input pyboxes into rects so that any rectangles will be
     // properly deduped in the resulting output.
     for (long i = 0; i < count; ++i)
-        rects.push_back(extract<rectangle>(pyboxes[i]));
+        rects.push_back(pyboxes[i].cast<rectangle>());
     // Find candidate objects
     find_candidate_object_locations(image, rects, kvals, min_size, max_merging_iterations);
 
@@ -126,55 +127,54 @@ inline void find_candidate_object_locations_py (
 
 // ----------------------------------------------------------------------------------------
 
-void bind_object_detection()
+void bind_object_detection(py::module& m)
 {
-    using boost::python::arg;
     {
     typedef simple_object_detector_training_options type;
-    class_<type>("simple_object_detector_training_options",
+    py::class_<type>(m, "simple_object_detector_training_options",
         "This object is a container for the options to the train_simple_object_detector() routine.")
-        .add_property("be_verbose", &type::be_verbose,
-                                    &type::be_verbose,
+        .def(py::init())
+        .def_readwrite("be_verbose", &type::be_verbose,
 "If true, train_simple_object_detector() will print out a lot of information to the screen while training.")
-        .add_property("add_left_right_image_flips", &type::add_left_right_image_flips,
-                                                    &type::add_left_right_image_flips,
+        .def_readwrite("add_left_right_image_flips", &type::add_left_right_image_flips,
 "if true, train_simple_object_detector() will assume the objects are \n\
 left/right symmetric and add in left right flips of the training \n\
 images.  This doubles the size of the training dataset.")
-        .add_property("detection_window_size", &type::detection_window_size,
-                                               &type::detection_window_size,
+        .def_readwrite("detection_window_size", &type::detection_window_size,
                                                "The sliding window used will have about this many pixels inside it.")
-        .add_property("C", &type::C,
-                           &type::C,
+        .def_readwrite("C", &type::C,
 "C is the usual SVM C regularization parameter.  So it is passed to \n\
 structural_object_detection_trainer::set_c().  Larger values of C \n\
 will encourage the trainer to fit the data better but might lead to \n\
 overfitting.  Therefore, you must determine the proper setting of \n\
 this parameter experimentally.")
-        .add_property("epsilon", &type::epsilon,
-                                 &type::epsilon,
+        .def_readwrite("epsilon", &type::epsilon,
 "epsilon is the stopping epsilon.  Smaller values make the trainer's \n\
 solver more accurate but might take longer to train.")
-        .add_property("num_threads", &type::num_threads,
-                                     &type::num_threads,
+        .def_readwrite("num_threads", &type::num_threads,
 "train_simple_object_detector() will use this many threads of \n\
 execution.  Set this to the number of CPU cores on your machine to \n\
-obtain the fastest training speed.");
+obtain the fastest training speed.")
+        .def_readwrite("upsample_limit", &type::upsample_limit,
+"train_simple_object_detector() will upsample images if needed \n\
+no more than upsample_limit times. Value 0 will forbid trainer to \n\
+upsample any images. If trainer is unable to fit all boxes with \n\
+required upsample_limit, exception will be thrown. Higher values \n\
+of upsample_limit exponentially increases memory requiremens. \n\
+Values higher than 2 (default) are not recommended.");
     }
     {
     typedef simple_test_results type;
-    class_<type>("simple_test_results")
-        .add_property("precision", &type::precision)
-        .add_property("recall", &type::recall)
-        .add_property("average_precision", &type::average_precision)
+    py::class_<type>(m, "simple_test_results")
+        .def_readwrite("precision", &type::precision)
+        .def_readwrite("recall", &type::recall)
+        .def_readwrite("average_precision", &type::average_precision)
         .def("__str__", &::print_simple_test_results);
     }
 
     // Here, kvals is actually the result of linspace(start, end, num) and it is different from kvals used
     // in find_candidate_object_locations(). See dlib/image_transforms/segment_image_abstract.h for more details.
-    def("find_candidate_object_locations", find_candidate_object_locations_py,
-            (arg("image"), arg("rects"), arg("kvals")=boost::python::make_tuple(50, 200, 3),
-             arg("min_size")=20, arg("max_merging_iterations")=50),
+    m.def("find_candidate_object_locations", find_candidate_object_locations_py, py::arg("image"), py::arg("rects"), py::arg("kvals")=py::make_tuple(50, 200, 3), py::arg("min_size")=20, py::arg("max_merging_iterations")=50,
 "Returns found candidate objects\n\
 requires\n\
     - image == an image object which is a numpy ndarray\n\
@@ -210,11 +210,11 @@ ensures\n\
       that:\n\
         - #rects[i] != rects[j]");
 
-    def("get_frontal_face_detector", get_frontal_face_detector,
+    m.def("get_frontal_face_detector", get_frontal_face_detector,
         "Returns the default face detector");
 
-    def("train_simple_object_detector", train_simple_object_detector,
-        (arg("dataset_filename"), arg("detector_output_filename"), arg("options")),
+    m.def("train_simple_object_detector", train_simple_object_detector,
+        py::arg("dataset_filename"), py::arg("detector_output_filename"), py::arg("options"),
 "requires \n\
     - options.C > 0 \n\
 ensures \n\
@@ -228,8 +228,8 @@ ensures \n\
       way to train a basic object detector.   \n\
     - The trained object detector is serialized to the file detector_output_filename.");
 
-    def("train_simple_object_detector", train_simple_object_detector_on_images_py,
-        (arg("images"), arg("boxes"), arg("options")),
+    m.def("train_simple_object_detector", train_simple_object_detector_on_images_py,
+        py::arg("images"), py::arg("boxes"), py::arg("options"),
 "requires \n\
     - options.C > 0 \n\
     - len(images) == len(boxes) \n\
@@ -244,9 +244,9 @@ ensures \n\
       way to train a basic object detector.   \n\
     - The trained object detector is returned.");
 
-    def("test_simple_object_detector", test_simple_object_detector,
+    m.def("test_simple_object_detector", test_simple_object_detector,
             // Please see test_simple_object_detector for the reason upsampling_amount is -1
-            (arg("dataset_filename"), arg("detector_filename"), arg("upsampling_amount")=-1),
+        py::arg("dataset_filename"), py::arg("detector_filename"), py::arg("upsampling_amount")=-1,
             "requires \n\
                 - Optionally, take the number of times to upsample the testing images (upsampling_amount >= 0). \n\
              ensures \n\
@@ -263,8 +263,8 @@ ensures \n\
                   metrics. "
         );
 
-    def("test_simple_object_detector", test_simple_object_detector_with_images_py,
-            (arg("images"), arg("boxes"), arg("detector"), arg("upsampling_amount")=0),
+    m.def("test_simple_object_detector", test_simple_object_detector_with_images_py,
+            py::arg("images"), py::arg("boxes"), py::arg("detector"), py::arg("upsampling_amount")=0,
             "requires \n\
                - len(images) == len(boxes) \n\
                - images should be a list of numpy matrices that represent images, either RGB or grayscale. \n\
@@ -282,9 +282,9 @@ ensures \n\
                  metrics. "
     );
 
-    def("test_simple_object_detector", test_simple_object_detector_py_with_images_py,
+    m.def("test_simple_object_detector", test_simple_object_detector_py_with_images_py,
             // Please see test_simple_object_detector_py_with_images_py for the reason upsampling_amount is -1
-            (arg("images"), arg("boxes"), arg("detector"), arg("upsampling_amount")=-1),
+            py::arg("images"), py::arg("boxes"), py::arg("detector"), py::arg("upsampling_amount")=-1,
             "requires \n\
                - len(images) == len(boxes) \n\
                - images should be a list of numpy matrices that represent images, either RGB or grayscale. \n\
@@ -302,13 +302,13 @@ ensures \n\
     );
     {
     typedef simple_object_detector type;
-    class_<type>("fhog_object_detector",
+    py::class_<type, std::shared_ptr<type>>(m, "fhog_object_detector",
         "This object represents a sliding window histogram-of-oriented-gradients based object detector.")
-        .def("__init__", make_constructor(&load_object_from_file<type>),  
+        .def(py::init(&load_object_from_file<type>),
 "Loads an object detector from a file that contains the output of the \n\
 train_simple_object_detector() routine or a serialized C++ object of type\n\
 object_detector<scan_fhog_pyramid<pyramid_down<6>>>.")
-        .def("__call__", run_detector_with_upscale2, (arg("image"), arg("upsample_num_times")=0),
+        .def("__call__", run_detector_with_upscale2, py::arg("image"), py::arg("upsample_num_times")=0,
 "requires \n\
     - image is a numpy ndarray containing either an 8bit grayscale or RGB \n\
       image. \n\
@@ -317,10 +317,8 @@ ensures \n\
     - This function runs the object detector on the input image and returns \n\
       a list of detections.   \n\
     - Upsamples the image upsample_num_times before running the basic \n\
-      detector.  If you don't know how many times you want to upsample then \n\
-      don't provide a value for upsample_num_times and an appropriate \n\
-      default will be used.")
-        .def("run", run_rect_detector, (arg("image"), arg("upsample_num_times")=0),
+      detector.")
+       .def("run", run_rect_detector, py::arg("image"), py::arg("upsample_num_times")=0, py::arg("adjust_threshold")=0.0,
 "requires \n\
     - image is a numpy ndarray containing either an 8bit grayscale or RGB \n\
       image. \n\
@@ -329,20 +327,29 @@ ensures \n\
     - This function runs the object detector on the input image and returns \n\
       a tuple of (list of detections, list of scores, list of weight_indices).   \n\
     - Upsamples the image upsample_num_times before running the basic \n\
-      detector.  If you don't know how many times you want to upsample then \n\
-      don't provide a value for upsample_num_times and an appropriate \n\
-      default will be used.")
-        .def("save", save_simple_object_detector, (arg("detector_output_filename")), "Save a simple_object_detector to the provided path.")
-        .def_pickle(serialize_pickle<type>());
+      detector.")
+       .def_static("run_multiple", run_multiple_rect_detectors, py::arg("detectors"),  py::arg("image"), py::arg("upsample_num_times")=0, py::arg("adjust_threshold")=0.0,
+"requires \n\
+    - detectors is a list of detectors. \n\
+    - image is a numpy ndarray containing either an 8bit grayscale or RGB \n\
+      image. \n\
+    - upsample_num_times >= 0 \n\
+ensures \n\
+    - This function runs the list of object detectors at once on the input image and returns \n\
+      a tuple of (list of detections, list of scores, list of weight_indices).   \n\
+    - Upsamples the image upsample_num_times before running the basic \n\
+      detector.")
+           .def("save", save_simple_object_detector, py::arg("detector_output_filename"), "Save a simple_object_detector to the provided path.")
+           .def(py::pickle(&getstate<type>, &setstate<type>));
     }
     {
     typedef simple_object_detector_py type;
-    class_<type>("simple_object_detector",
+    py::class_<type, std::shared_ptr<type>>(m, "simple_object_detector",
         "This object represents a sliding window histogram-of-oriented-gradients based object detector.")
-        .def("__init__", make_constructor(&load_object_from_file<type>),
+        .def(py::init(&load_object_from_file<type>),
 "Loads a simple_object_detector from a file that contains the output of the \n\
 train_simple_object_detector() routine.")
-        .def("__call__", &type::run_detector1, (arg("image"), arg("upsample_num_times")),
+        .def("__call__", &type::run_detector1, py::arg("image"), py::arg("upsample_num_times"),
 "requires \n\
     - image is a numpy ndarray containing either an 8bit grayscale or RGB \n\
       image. \n\
@@ -354,15 +361,15 @@ ensures \n\
       detector.  If you don't know how many times you want to upsample then \n\
       don't provide a value for upsample_num_times and an appropriate \n\
       default will be used.")
-        .def("__call__", &type::run_detector2, (arg("image")),
+        .def("__call__", &type::run_detector2, py::arg("image"),
 "requires \n\
     - image is a numpy ndarray containing either an 8bit grayscale or RGB \n\
       image. \n\
 ensures \n\
     - This function runs the object detector on the input image and returns \n\
       a list of detections.")
-        .def("save", save_simple_object_detector_py, (arg("detector_output_filename")), "Save a simple_object_detector to the provided path.")
-        .def_pickle(serialize_pickle<type>());
+        .def("save", save_simple_object_detector_py, py::arg("detector_output_filename"), "Save a simple_object_detector to the provided path.")
+        .def(py::pickle(&getstate<type>, &setstate<type>));
     }
 }
 
